package com.jjt.calc.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.jjt.common.annotation.Log;
import com.jjt.common.core.controller.BaseController;
import com.jjt.common.core.domain.AjaxResult;
import com.jjt.common.enums.BusinessType;
import com.jjt.calc.domain.TwinCalcHour;
import com.jjt.calc.service.ITwinCalcHourService;
import com.jjt.common.utils.poi.ExcelUtil;
import com.jjt.common.core.page.TableDataInfo;

/**
 * 1小时统计数据Controller
 *
 * @author wukai
 * @date 2025-01-18
 */
@Api(tags="1小时统计数据")
@RestController
@RequestMapping("/calc/calcHour")
public class TwinCalcHourController extends BaseController{
    @Resource
    private ITwinCalcHourService twinCalcHourService;

    /**
     * 查询1小时统计数据列表
     */
    @ApiOperation("查询1小时统计数据列表")
    @PreAuthorize("@ss.hasPermi('calc:calcHour:list')")
    @GetMapping("/list")
    public TableDataInfo list(TwinCalcHour twinCalcHour)
    {
        startPage();
        List<TwinCalcHour> list = twinCalcHourService.selectTwinCalcHourList(twinCalcHour);
        return getDataTable(list);
    }

    /**
     * 导出1小时统计数据列表
     */
    @ApiOperation("导出1小时统计数据列表")
    @PreAuthorize("@ss.hasPermi('calc:calcHour:export')")
    @Log(title = "1小时统计数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TwinCalcHour twinCalcHour)
    {
        List<TwinCalcHour> list = twinCalcHourService.selectTwinCalcHourList(twinCalcHour);
        ExcelUtil<TwinCalcHour> util = new ExcelUtil<TwinCalcHour>(TwinCalcHour.class);
        util.exportExcel(response, list, "1小时统计数据数据");
    }

    /**
     * 获取1小时统计数据详细信息
     */
    @ApiOperation("获取1小时统计数据详细信息")
    @PreAuthorize("@ss.hasPermi('calc:calcHour:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(twinCalcHourService.selectTwinCalcHourById(id));
    }

    /**
     * 新增1小时统计数据
     */
    @ApiOperation("新增1小时统计数据")
    @PreAuthorize("@ss.hasPermi('calc:calcHour:add')")
    @Log(title = "1小时统计数据", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TwinCalcHour twinCalcHour)
    {
        return toAjax(twinCalcHourService.insertTwinCalcHour(twinCalcHour));
    }

    /**
     * 修改1小时统计数据
     */
    @ApiOperation("修改1小时统计数据")
    @PreAuthorize("@ss.hasPermi('calc:calcHour:edit')")
    @Log(title = "1小时统计数据", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TwinCalcHour twinCalcHour)
    {
        return toAjax(twinCalcHourService.updateTwinCalcHour(twinCalcHour));
    }

    /**
     * 删除1小时统计数据
     */
    @ApiOperation("删除1小时统计数据")
    @PreAuthorize("@ss.hasPermi('calc:calcHour:remove')")
    @Log(title = "1小时统计数据", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(twinCalcHourService.deleteTwinCalcHourByIds(ids));
    }
}
